@@ -1,5 +1,7 @@
 Game.ImportLibrary("system/scripts/scenario_original.lua")
 
+Game.DoFile("system/scripts/input_handling.lua")
+
 Scenario.tRandoHintPropIDs = {
     CAVE_1 = Blackboard.RegisterLUAProp("HINT_CAVE_1", "bool"),
     CAVE_2 = Blackboard.RegisterLUAProp("HINT_CAVE_2", "bool"),
@@ -14,6 +16,8 @@ Scenario.tRandoHintPropIDs = {
     SHIP_1 = Blackboard.RegisterLUAProp("HINT_SHIP_1", "bool")
 }
 
+Scenario.RandoTrueXRelease = Blackboard.RegisterLUAProp("X_RELEASE_TRUE", "bool")
+
 function Scenario.CheckRandoHint(ap_id, hint_id)
     local access_point = Game.GetActor(ap_id)
     local seen = Scenario.ReadFromBlackboard(Scenario.tRandoHintPropIDs[hint_id], false)
@@ -160,6 +164,7 @@ function Scenario.DisableGlobalTeleport(actor)
 
     local teleportal_id = Scenario.GetTeleportalID(actor)
     if Blackboard.GetProp("GAME_PROGRESS", teleportal_id) then return end
+    if Blackboard.GetProp("GAME_PROGRESS", "RandoMapSeen" .. actor.USABLE.sScenarioName) then return end
     Blackboard.SetProp("GAME_PROGRESS", "RandoTeleportWorldUnlocked", "b", true)
     Blackboard.SetProp("GAME_PROGRESS", "TeleportWorldUnlocked", "b", false)
 end
@@ -215,6 +220,26 @@ function Scenario.VisitAllTeleportScenarios()
     return true
 end
 
+function Scenario._UpdateProgressiveItemModels()
+    for name, actordef in pairs(Game.GetEntities()) do
+        local progressive_models = RandomizerPowerup.tProgressiveModels[actordef]
+        if progressive_models ~= nil then
+            for _, model in ipairs(progressive_models) do
+                if RandomizerPowerup.HasItem(model.item) then
+                    local pickup = Game.GetActor(name)
+                    pickup.MODELUPDATER.sModelAlias = model.alias
+                    pickup.MODELUPDATER:ForceUpdate()
+                    break
+                end
+            end
+        end
+    end
+end
+
+function Scenario.UpdateProgressiveItemModels()
+    Game.AddSF(0.1, "Scenario._UpdateProgressiveItemModels", "")
+end
+
 local original_onload = Scenario.OnLoadScenarioFinished
 function Scenario.OnLoadScenarioFinished()
     original_onload()
@@ -223,6 +248,12 @@ function Scenario.OnLoadScenarioFinished()
 
     if Scenario.VisitAllTeleportScenarios() then return end
 
+    Scenario.UpdateProgressiveItemModels()
+
+    Blackboard.SetProp("GAME_PROGRESS", "RandoMapSeen" .. CurrentScenarioID, "b", true)
+
+    Game.AddSF(0, "Scenario.CheckDebugInputs", "")
+
     local teleportal_id = Blackboard.GetProp("GAME_PROGRESS", "RandoUnlockTeleportal")
     if teleportal_id == nil then return end
     local platform = Game.GetActor(teleportal_id)
@@ -239,3 +270,57 @@ function Scenario.CheckArtifactsObtained(actor, diag)
         end
     end
 end
+
+local save_names = {
+    'PRP_CV_AccessPoint001',
+    'PRP_CV_AccessPoint002',
+    'PRP_CV_MapStation001',
+    'PRP_CV_SaveStation001',
+    'PRP_CV_SaveStation002',
+    'PRP_CV_SaveStation003',
+    'PRP_CV_SaveStation004',
+    'accesspoint',
+    'accesspoint_000',
+    'accesspoint_001',
+    'maproom',
+    'maproom_000',
+    'savestation',
+    'savestation_000',
+    'savestation_001',
+    'savestation_002',
+}
+
+function Scenario.IsSaveStation(actor)
+    for _, name in ipairs(save_names) do
+        if actor.sName == name then
+            return true
+        end
+    end
+    return false
+end
+
+function Scenario.CheckWarpToStart(actor)
+    if not Scenario.IsSaveStation(actor) then return end
+    if not Init.bWarpToStart then return end
+    
+    Input.LogInputs()
+    if Input.CheckInputs("ZL", "ZR") then
+        Scenario.WarpToStart()
+    end
+end
+
+function Scenario.WarpToStart()
+    Game.LoadScenario("c10_samus", Init.sStartingScenario, Init.sStartingActor, "", 1)
+end
+
+function Scenario.CheckDebugInputs()
+    local delay = 0
+
+    if Input.CheckInputs("ZL", "ZR", "DPAD_UP") then
+        delay = 0.5
+        Game.ReinitPlayerFromBlackboard()
+        -- RandomizerPowerup.DisableInput()
+        -- RandomizerPowerup.ChangeSuit()
+    end
+    Game.AddSF(delay, "Scenario.CheckDebugInputs", "")
+end
